# Generated by Django 4.2.7 on 2026-02-17 07:25

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nai_security", "0003_whitelisteduser"),
    ]

    operations = [
        migrations.AddField(
            model_name="securitysettings",
            name="axes_attempt_expiry_enabled",
            field=models.BooleanField(
                default=False,
                help_text="Each failed attempt expires independently (requires cooloff > 0)",
            ),
        ),
        migrations.AddField(
            model_name="securitysettings",
            name="axes_cooloff_minutes",
            field=models.PositiveIntegerField(
                default=0,
                help_text="Minutes before locked account auto-unlocks (0 = permanent lock until manual reset)",
                validators=[django.core.validators.MaxValueValidator(1440)],
            ),
        ),
    ]
